/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using Gtk;
using Glade;
using MovieDatabase;
using Gnome;
using Gdk;
using Util;

public class Window_Main 
{
    // Menu Items
    [Widget] Gtk.MenuItem MainMenu_View_MenuItem;
    [Widget] Gtk.Window Window_Main_GtkWindow;
    
    // Other widgets
    [Widget] Gtk.Entry Search_GtkEntry;
    [Widget] Gtk.Button Search_GtkButton;
    [Widget] Gtk.Statusbar Status_GtkStatusbar;
   
    [Widget] Gtk.ScrolledWindow DataContainer_GtkScrolledWindow;
    [Widget] Gtk.ScrolledWindow MovieContainer_GtkScrolledWindow;
    
    [Widget] Gtk.MenuItem Import_MenuItem;
    [Widget] Gtk.MenuItem Export_MenuItem;
    
    [Widget] Gtk.Button Remove_GtkButton;
    [Widget] Gtk.Button Return_GtkButton;

    // Borrower Control
    [Widget] Gtk.Alignment ComboAlignment_GtkAlignment;
    Gtk.ComboCached Borrower_GtkComboBoxEntry;

    // Flag Control
    [Widget] Gtk.CheckButton Lost_GtkCheckButton;
    [Widget] Gtk.CheckButton Rare_GtkCheckButton;
    [Widget] Gtk.CheckButton Damaged_GtkCheckButton;
    [Widget] Gtk.CheckButton Copy_GtkCheckButton;
    [Widget] Gtk.CheckButton Favourite_GtkCheckButton;
    [Widget] Gtk.CheckButton HardDrive_GtkCheckButton;

    // Images
    [Widget] Gtk.Image Disc1_GtkImage;
    [Widget] Gtk.Image Disc2_GtkImage;
    [Widget] Gtk.Image Disc3_GtkImage;
    [Widget] Gtk.Image Disc4_GtkImage;
    
    [Widget] Gtk.Label Title_GtkLabel;
    [Widget] Gtk.Label BorrowedFor_GtkLabel;

    public static DataView MovieDataView;
    public static MovieView MovieMovieView;

    // Gconf
    
    private const string GConf_Width = "/apps/monotheka/mainwindow/width" ;
    private const int GConf_Default_Width = 700;
    private const string GConf_Height = "/apps/monotheka/mainwindow/height" ;
    private const int GConf_Default_Height = 500;

    private const string GConf_Columns = "/apps/monotheka/mainwindow/dbcolumns" ;
    private const string GConf_Default_Columns = "Title,Director,Flags,No";
    
    
    // *** Constructor 
    public Window_Main (string[] args)
    {
        Application.Init();
       
        Glade.XML gxml = new Glade.XML (null, "MainWindow.glade", "Window_Main_GtkWindow", null);
        gxml.Autoconnect (this);

        // Set icon
        // FIXME: This should be move to some common icon container class...
        Window_Main_GtkWindow.Icon = new Pixbuf(null, "windowicon.png");

        // Serves as settings now
        ColumnsRequest CR = new ColumnsRequest((string) GConfRegistry.Get(GConf_Columns, GConf_Default_Columns));

        // Borrower Widget
        Borrower_GtkComboBoxEntry = new Gtk.ComboCached(Global.Borrowers_Cache);
        Borrower_GtkComboBoxEntry.Entry.Changed += new EventHandler(On_BorrowerChanged);
        Borrower_GtkComboBoxEntry.Entry.Activated += new EventHandler(On_BorrowerActivated);
        Borrower_GtkComboBoxEntry.Sensitive = false;
        ComboAlignment_GtkAlignment.Add(Borrower_GtkComboBoxEntry);
        
        // Plugin Menu
        Menu ImportPlugins_Menu = new Menu();
        Menu ExportPlugins_Menu = new Menu();

        // Import plugins
        foreach (PluginLib.ImportPlugin __P in PluginManager.ImportPlugins) {

            MenuItem __I = new MenuItem(((PluginLib.Plugin)__P).GetName());
            __I.Activated += new EventHandler(On_ImportPlugin);
            ImportPlugins_Menu.Append(__I);
        }

        // Export plugins
        foreach (PluginLib.ExportPlugin __P in PluginManager.ExportPlugins) {

            MenuItem __I = new MenuItem(((PluginLib.Plugin)__P).GetName());
            __I.Activated += new EventHandler(On_ExportPlugin);
            ExportPlugins_Menu.Append(__I);
        }

        // Attach
        Import_MenuItem.Submenu = ImportPlugins_Menu;
        Export_MenuItem.Submenu = ExportPlugins_Menu;
                
        // Create ViewMenu Items
        Gtk.Menu MainMenu_View_Menu = (Gtk.Menu) MainMenu_View_MenuItem.Submenu; // new Menu();

        for (int __k = MovieDB.Fields.Count-1; __k >= 0; __k--) {
            if (((DataField)MovieDB.Fields[__k]).Columnable) {
                Gtk.CheckMenuItem __ViewItem = new CheckMenuItem(((DataField)MovieDB.Fields[__k]).Name);
                if (CR.Columns[__k] == true) __ViewItem.Active = true; else __ViewItem.Active = false;

                __ViewItem.Toggled += new EventHandler(On_ViewItem_Clicked);
                
                MainMenu_View_Menu.Prepend(__ViewItem);
            }
        }
        
        // Create the main Database view
        MovieDataView = new DataView(CR);
        DataContainer_GtkScrolledWindow.Add(MovieDataView);

        // Create the movie-details view
        // FIXME The borrower combo box is used to get the widgets color
        // and set it as default for MovieView html view. kind of clumsy.
        MovieMovieView = new MovieView(Rc.GetStyle(Borrower_GtkComboBoxEntry), Disc1_GtkImage, Disc2_GtkImage, Disc3_GtkImage, Disc4_GtkImage, Title_GtkLabel, BorrowedFor_GtkLabel);
        MovieContainer_GtkScrolledWindow.Add(MovieMovieView);
    
        // Events
        MovieDataView.MovieSelected += new DataView.SelectedMovieHandler(On_MovieSelected);
        MovieDataView.MovieLeaved += new DataView.SelectedMovieHandler(On_MovieLeaved);
        MovieDataView.RowActivated += new RowActivatedHandler(On_RowActivated);
        Window_Main_GtkWindow.SizeAllocated += new SizeAllocatedHandler (On_SizeAllocated);

        // Restore state
        int __Width = (int) GConfRegistry.Get(GConf_Width, GConf_Default_Width);
        int __Height = (int) GConfRegistry.Get(GConf_Height, GConf_Default_Height);
        Window_Main_GtkWindow.SetDefaultSize(__Width, __Height);

        // Load all the settings
        GConfMonitor.LoadSettings();
        
        // List database
        if (!MovieDB.FreshDatabase) Status_GtkStatusbar.Push(1, "Welcome to Monotheka! " + Convert.ToString(MovieDataView.RefreshView(false)) + " movies found.");
        else Status_GtkStatusbar.Push(1, "Welcome to Monotheka! Database not found, fresh database created.");

                
        Window_Main_GtkWindow.ShowAll();
        if (Upgrades.UpgradedTo4) new Util.Windows.OKDialogWindow(Upgrades.Msg_UpgradedTo4, Window_Main_GtkWindow, MessageType.Info);
        
        // Start monitoring the GConf regisrty
        GConfMonitor.Init();
        
        Application.Run();
    }
  



    /* Connect the Signals defined in Glade */
    public void OnWindowDeleteEvent (object o, DeleteEventArgs args) 
    {
        On_Quit(null, null);
        args.RetVal = true;
    }
  
    /* Settings Menu */
    public void On_Settings (object o, EventArgs args) 
    {
        new Window_Properties();
    }

    // *** Remove button clicked
    public void On_RemoveMovie (object o, EventArgs args) 
    {
        if (MovieDataView.SelectedIter.Stamp != 0) {
            // Display dialog
            MessageDialog md = new MessageDialog(Window_Main_GtkWindow, DialogFlags.DestroyWithParent, 
                                                 MessageType.Question, ButtonsType.YesNo, "Are you sure that you want to delete " + MovieMovieView.DisplayedMovie.Data[(int)MovieField.TITLE] + " from your database?");
            
            if ((ResponseType)md.Run() == ResponseType.Yes)
                {
                    md.Destroy();
                    MovieDB.RemoveMovie(MovieMovieView.DisplayedMovie);
                    Status_GtkStatusbar.Push(1, MovieMovieView.DisplayedMovie.Data[(int)MovieField.TITLE] + " removed from database.");
                    if (!MovieDataView.RemoveFromTreeView(MovieDataView.SelectedIter)) {
                        // Last position selected. Need to reset MovieView
                        MovieMovieView.Reset();
                        Deactivate_MovieWidgets();
                        
                        }; 
                }
            else md.Destroy();
            MovieDataView.GrabFocus();
        }
        
    }

    public void Activate_MovieWidgets()
    {
        Remove_GtkButton.Sensitive = true;
       
        Lost_GtkCheckButton.Sensitive = true;
        Rare_GtkCheckButton.Sensitive = true;
        Damaged_GtkCheckButton.Sensitive = true;
        Copy_GtkCheckButton.Sensitive = true;
        Favourite_GtkCheckButton.Sensitive = true;
        HardDrive_GtkCheckButton.Sensitive = true;
        
        Borrower_GtkComboBoxEntry.Sensitive = true;
        

    }

    public void Deactivate_MovieWidgets()
    {
        Remove_GtkButton.Sensitive = false;
        Return_GtkButton.Sensitive = false;
        
        Lost_GtkCheckButton.Sensitive = false;
        Rare_GtkCheckButton.Sensitive = false;
        Damaged_GtkCheckButton.Sensitive = false;
        Copy_GtkCheckButton.Sensitive = false;
        Favourite_GtkCheckButton.Sensitive = false;
        HardDrive_GtkCheckButton.Sensitive = false;
        
        Borrower_GtkComboBoxEntry.Sensitive = false;
    }
    


    // Search button pressed
    public void On_Search (object o, EventArgs args) 
    {
        
        // TODO: Some spaces trimming
        
        if (Search_GtkEntry.Text != null) 
            {
                On_MovieLeaved(this, null);
                
                SearchRequest __SR = new SearchRequest(Search_GtkEntry.Text);
                Status_GtkStatusbar.Push(1, Convert.ToString(MovieDataView.RefreshView(__SR) + " movies found."));
                __SR = null;
                
                // Temporarily
                MovieMovieView.Reset();
                Deactivate_MovieWidgets();

            }

    }

    public void On_MovieSelected(object sender, string ID)
    {
        Movie M = MovieDB.GetSingleRecord(ID);
        MovieMovieView.LoadMovie(M);
        MovieMovieView.RenderData();
        
        // Set the borrower
        Borrower_GtkComboBoxEntry.Entry.Text = M.Data[(int)MovieField.BORROWER];
        // Get the flags
        FlagsList __FList = new FlagsList(M.Data[(int)MovieField.FLAGS]);
        
        // Reset widgets
        Lost_GtkCheckButton.Active = false;
        Damaged_GtkCheckButton.Active = false;
        Rare_GtkCheckButton.Active = false;
        Copy_GtkCheckButton.Active = false;
        Favourite_GtkCheckButton.Active = false;
        HardDrive_GtkCheckButton.Active = false;
        
        
        if (__FList.IsPresent(MovieFlag.LOST)) Lost_GtkCheckButton.Active = true;
        if (__FList.IsPresent(MovieFlag.DAMAGED)) Damaged_GtkCheckButton.Active = true;
        if (__FList.IsPresent(MovieFlag.RARE)) Rare_GtkCheckButton.Active = true;
        if (__FList.IsPresent(MovieFlag.COPYNEEDED)) Copy_GtkCheckButton.Active = true;
        if (__FList.IsPresent(MovieFlag.FAVOURITE)) Favourite_GtkCheckButton.Active = true;
        if (__FList.IsPresent(MovieFlag.HARDDRIVE)) HardDrive_GtkCheckButton.Active = true;
            
        // Activeate remove button etc.
        Activate_MovieWidgets();

        // Let's see if we need to activate "return" button
        if (M.Data[(int)MovieField.BORROWER] != "") Return_GtkButton.Sensitive = true;
        else  Return_GtkButton.Sensitive = false;
    }

    
    public void On_MovieLeaved(object sender, string ID)
    {
        
        if (MovieMovieView.DisplayedMovie != null) 
            if (MovieMovieView.MovieModified) {
                // Fill the movie
                MovieDB.UpdateMovie(MovieMovieView.DisplayedMovie, new ColumnsRequest(MovieField.BORROWER, MovieField.FLAGS, MovieField.BORROWEDDATE));
                if (Borrower_GtkComboBoxEntry.Entry.Text != "")  Borrower_GtkComboBoxEntry.AddCurrent();
                
                
            }
        
    }
    

    
    public void On_About (object o, EventArgs args)
    {
        new Window_About(Window_Main_GtkWindow);
    }


    public void On_ViewItem_Clicked (object o, EventArgs args)
    {
        Gtk.CheckMenuItem __MI = (Gtk.CheckMenuItem) o;
        Gtk.Label __Label = (Gtk.Label) __MI.Child;
        
        // Find column (MovieField) by name
        int __i = 0;
        foreach (DataField __DF in MovieDB.Fields) {
            if (__DF.Name == __Label.Text) {
                if (__MI.Active == true) {
                    // Show Column
                    MovieDataView.ShowColumn((MovieField)__i);
                    return;
                }
                else {
                    // Hide Column
                    MovieDataView.HideColumn((MovieField)__i);
                    return;
                }
            }
            __i++;
        }
        
        
    }

    public void On_BorrowerChanged (object o, EventArgs args)
    {
        
        if (Borrower_GtkComboBoxEntry.Entry.Text == MovieMovieView.DisplayedMovie.Data[(int)MovieField.BORROWER] || Borrower_GtkComboBoxEntry.Entry.Text == "") return;

        string __NewValue = Borrower_GtkComboBoxEntry.Entry.Text;
        if (__NewValue == "<empty>" ) { 
            __NewValue = "" ; 
            Borrower_GtkComboBoxEntry.Entry.Text = "";
        }
        
        MovieMovieView.MovieModified = true;
        
        if (__NewValue == "") { 
            // Return mode
            Return_GtkButton.Sensitive = false; 
            MovieMovieView.DisplayedMovie.Data[(int)MovieField.BORROWEDDATE] = String.Empty;
        } else {
            Return_GtkButton.Sensitive = true; 
            MovieMovieView.DisplayedMovie.Data[(int)MovieField.BORROWEDDATE] = DateTime.Now.ToFileTime().ToString();
        }

        MovieMovieView.DisplayedMovie.Data[(int)MovieField.BORROWER] = __NewValue;
        MovieMovieView.Refresh();
        MovieDataView.FillRowWithMovie(MovieMovieView.DisplayedMovie);
        Status_GtkStatusbar.Push(1, MovieMovieView.DisplayedMovie.Data[(int)MovieField.TITLE] + " borrowed to " + __NewValue);
        
    }

    public void On_BorrowerActivated(object o, EventArgs args)
    {
        MovieDataView.GrabFocus();
    }

    public void On_MovieFlagClicked(object o, EventArgs args)
    {
        FlagsList _FL = new FlagsList(MovieMovieView.DisplayedMovie.Data[(int)MovieField.FLAGS]);
        Gtk.CheckButton __FlagBtn = (Gtk.CheckButton) o;

        if (__FlagBtn == Lost_GtkCheckButton) {
            if (!Lost_GtkCheckButton.Active == true) _FL.Add(MovieFlag.LOST);
            else _FL.Remove(MovieFlag.LOST);
            }
        
        if (__FlagBtn == Rare_GtkCheckButton) {
            if (!Rare_GtkCheckButton.Active == true) _FL.Add(MovieFlag.RARE);
            else _FL.Remove(MovieFlag.RARE);
            }
        
        if (__FlagBtn == Copy_GtkCheckButton) {
            if (!Copy_GtkCheckButton.Active == true) _FL.Add(MovieFlag.COPYNEEDED);
            else _FL.Remove(MovieFlag.COPYNEEDED);
            }
        
        if (__FlagBtn == Damaged_GtkCheckButton) {
            if (!Damaged_GtkCheckButton.Active == true) _FL.Add(MovieFlag.DAMAGED);
            else _FL.Remove(MovieFlag.DAMAGED);
            }

        if (__FlagBtn == Favourite_GtkCheckButton) {
            if (!Favourite_GtkCheckButton.Active == true) _FL.Add(MovieFlag.FAVOURITE);
            else _FL.Remove(MovieFlag.FAVOURITE);
        }

        if (__FlagBtn == HardDrive_GtkCheckButton) {
            if (!HardDrive_GtkCheckButton.Active == true) _FL.Add(MovieFlag.HARDDRIVE);
            else _FL.Remove(MovieFlag.HARDDRIVE);
        }


        
        MovieMovieView.MovieModified = true;
        MovieMovieView.DisplayedMovie.Data[(int)MovieField.FLAGS] = _FL.ToString();
        MovieDataView.FillRowWithMovie(MovieMovieView.DisplayedMovie);
        MovieDataView.GrabFocus();
                
    }


    public void On_AddMovie (object o, EventArgs args)
    {
        EditMovie_Window __NewMovieWindow = new EditMovie_Window();
        Movie __NewMovie = __NewMovieWindow.Run();

        if (__NewMovie != null) { 
            
            // Clear search
            if (Search_GtkEntry.Text != null && Search_GtkEntry.Text != "") {
                Search_GtkEntry.Text = "";
                MovieDataView.RefreshView(null);
            }

            __NewMovie = MovieDB.AddMovie(__NewMovie); // To obtain the ID
            TreeIter __NewIter = (TreeIter) MovieDataView.AddToTreeView (__NewMovie, null);
            MovieDataView.SetSelection(__NewIter);
            
            Status_GtkStatusbar.Push(1, __NewMovie.Data[(int)MovieField.TITLE] + " has been added to the database.");
            MovieDataView.GrabFocus();
            }


    }

    public void On_RowActivated (object o, RowActivatedArgs args)
    {
        EditMovie_Window __EditMovieWindow = new EditMovie_Window(MovieMovieView.DisplayedMovie);
        Movie __EditedMovie = __EditMovieWindow.Run();

        if (__EditedMovie != null) {
            ColumnsRequest __CR = new ColumnsRequest();
            __CR.AllColumns();
            MovieDB.UpdateMovie(__EditedMovie, __CR);
            MovieMovieView.LoadMovie(__EditedMovie);
            MovieMovieView.RenderData();
            MovieDataView.FillRowWithMovie(MovieMovieView.DisplayedMovie);
            Status_GtkStatusbar.Push(1, Convert.ToString(MovieMovieView.DisplayedMovie.Data[(int)MovieField.TITLE]) + " modified in the database");
            
            
        }


    }
       
    public void On_Menu_Plugins (object o, EventArgs args)
    {
        new Window_Plugins();
    }

    
    public void On_Quit (object o, EventArgs args)
    {
        On_MovieLeaved(this, null);

        // Save the gconf settings
        // Config.Set("

        Application.Quit ();
    }

    public void On_ExportPlugin (object o, EventArgs args)
    {
        MenuItem __MI;
        Label __L;
        __MI = (MenuItem) o;
        __L = (Label) __MI.Child;
        
        foreach (PluginLib.ExportPlugin __Plugin in PluginManager.ExportPlugins) {
            if (__L.Text == __Plugin.GetName()) { 
                __Plugin.Run();
                return;
            }
        }
                
        
    }

    /* Return movie clicked */
    public void On_ReturnMovie(object o, EventArgs args)
    {
        Borrower_GtkComboBoxEntry.Entry.Text = "<empty>";
    }

    public void On_ImportPlugin (object o, EventArgs args)
    {
        MenuItem __MI;
        Label __L;
        __MI = (MenuItem) o;
        __L = (Label) __MI.Child;
        

        // Check plugin...
        foreach (PluginLib.ImportPlugin __Plugin in PluginManager.ImportPlugins) {
            if (__L.Text == __Plugin.GetName()) { 
                // Now, let's run the plugin
                if (__Plugin.Run()) {
                    new Window_Caching();
                    // Clear search
                    Search_GtkEntry.Text = "";
                    Status_GtkStatusbar.Push(1, "Import operation succeded.");
                    MovieMovieView.Reset();
                    MovieDataView.RefreshView(null);
                    // Hide button
                    Remove_GtkButton.Sensitive = false;
                    // FIXME: Selection? 
                };
                return;
            }
        }
    }    
        
    private void On_Clear (object o, EventArgs args)
    {
        // Warnin dialog
        MessageDialog md = new MessageDialog(Window_Main_GtkWindow, DialogFlags.DestroyWithParent, 
                                             MessageType.Warning, ButtonsType.YesNo, "You're about to COMPLETELY wipe out your movie database. All the movies will be deleted, but the cached combo values will remain untouched.\n\nDo you wan to proceed?");
        
        if ((ResponseType)md.Run() == ResponseType.Yes)
            {
                md.Destroy();
                MovieDB.ClearDatabase();
                Status_GtkStatusbar.Push(1, "Movie database cleared!");
                MovieDataView.RefreshView(false);
                MovieMovieView.Reset();
                Deactivate_MovieWidgets();
            }
        else md.Destroy();
    }
    
    
    
    
    private void On_SizeAllocated (object o, SizeAllocatedArgs args)
    {
        int __Width, __Height;
        
        Window_Main_GtkWindow.GetSize (out __Width, out __Height);
        
        GConfRegistry.Set (GConf_Width, __Width);
        GConfRegistry.Set (GConf_Height, __Height);
    }

    private void On_ShowBorrowed (object o, EventArgs args)
    {
        On_MovieLeaved(this, null);
        Search_GtkEntry.Text = "";
        
        SearchRequest __SR = new SearchRequest(new SearchElement((int)MovieField.BORROWER, SearchComparisonType.NOTEQUAL, "")); 
        Status_GtkStatusbar.Push(1, Convert.ToString(MovieDataView.RefreshView(__SR) + " borrowed movies found."));
        __SR = null;
        
        MovieMovieView.Reset();
        Deactivate_MovieWidgets();
    }

    private void On_ShowFavourite (object o, EventArgs args)
    {
        On_MovieLeaved(this, null);
        Search_GtkEntry.Text = "";
        
        SearchRequest __SR = new SearchRequest(new SearchElement((int)MovieField.FLAGS, SearchComparisonType.LIKE, "016"));
        Status_GtkStatusbar.Push(1, Convert.ToString(MovieDataView.RefreshView(__SR) + " favourite movies found."));
        __SR = null;
        
        MovieMovieView.Reset();
        Deactivate_MovieWidgets();
    }

    private void On_ShowDamagedLost (object o, EventArgs args)
    {
        On_MovieLeaved(this, null);
        Search_GtkEntry.Text = "";
        
        SearchRequest __SR = new SearchRequest(SearchOperand.OR); 
        __SR.Add(new SearchElement((int) MovieField.FLAGS, SearchComparisonType.LIKE, "002"));
        __SR.Add(new SearchElement((int) MovieField.FLAGS, SearchComparisonType.LIKE, "001"));
        Status_GtkStatusbar.Push(1, Convert.ToString(MovieDataView.RefreshView(__SR) + " damaged movies found."));
        __SR = null;
        
        MovieMovieView.Reset();
        Deactivate_MovieWidgets();
    }

    private void On_ShowHarddrive (object o, EventArgs args)
    {
        On_MovieLeaved(this, null);
        Search_GtkEntry.Text = "";
        
        SearchRequest __SR = new SearchRequest(SearchOperand.OR); 
        __SR.Add(new SearchElement((int) MovieField.FLAGS, SearchComparisonType.LIKE, "032"));
        Status_GtkStatusbar.Push(1, Convert.ToString(MovieDataView.RefreshView(__SR) + " movies on the harddrive found."));
        __SR = null;
        
        MovieMovieView.Reset();
        Deactivate_MovieWidgets();
    }


    private void On_ShowAll (object o, EventArgs args)
    {
        On_MovieLeaved(this, null);
        Search_GtkEntry.Text = "";
        
        Status_GtkStatusbar.Push(1, Convert.ToString(MovieDataView.RefreshView(null)) + " movies found. Showing all.");
        MovieMovieView.Reset();
        Deactivate_MovieWidgets();
        
    }


    private void On_BackupDatabase (object o, EventArgs args)
    {
                
        Util.Windows.FileSelectionDialogWindow __BackupLocation = new Util.Windows.FileSelectionDialogWindow(null, "Select a backup location");

        if (__BackupLocation.GetResult() != null) {
            // Backup the database
            // FIXME: Move this to some kind of file utils!
            Util.Debug.Message("Monotheka.exe", "MainWindow", "Backuping DB to: {0}" + __BackupLocation.GetResult());
            string __Dir = Gnome.User.DirGet() + "/monotheka";
            try {
                System.IO.File.Copy( __Dir + "/monotheka.db", __BackupLocation.GetResult(), true);
            } catch {
                // Message dialog
                new Util.Windows.OKDialogWindow("Database could not be copied to a specified location!",  Window_Main_GtkWindow, MessageType.Error);
            }

        }

        
    }
    
}

