/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using Gtk;
using Glade;
using MovieDatabase;
using Gnome;
using Gdk;
using Util;

public class Window_Properties {
    
    /* Database page */
    [Widget] Gtk.Window Window_Properties_GtkWindow;
    [Widget] Gtk.Entry Name_GtkEntry;
    [Widget] Gtk.Entry Owner_GtkEntry;
    [Widget] Gtk.Entry Contact_GtkEntry;
    [Widget] Gtk.TextView Comment_GtkTextView;
    
    /* Network page */
    [Widget] Gtk.RadioButton BrowserGNOME_GtkRadioButton;
    [Widget] Gtk.RadioButton BrowserCustom_GtkRadioButton;
    [Widget] Gtk.RadioButton ProxyGNOME_GtkRadioButton;
    [Widget] Gtk.RadioButton ProxyCustom_GtkRadioButton;
    [Widget] Gtk.Entry BrowserCommand_GtkEntry;
    [Widget] Gtk.Entry Server_GtkEntry;
    [Widget] Gtk.Entry Port_GtkEntry;
    [Widget] Gtk.Entry Username_GtkEntry;
    [Widget] Gtk.Entry Password_GtkEntry;
    
    /* Display page */
    [Widget] Gnome.ColorPicker Borrowed_GnomeColorPicker;
    [Widget] Gnome.ColorPicker OverBorrowed_GnomeColorPicker;
    [Widget] Gnome.ColorPicker New_GnomeColorPicker;

    [Widget] Gtk.Entry NewDays_GtkEntry;
    [Widget] Gtk.Entry OverBorrowedDays_GtkEntry;

    //////////////////////////////////////////////////
    
    /* Constructor */
    public Window_Properties() 
    {
        Glade.XML gxml = new Glade.XML (null, "PropertiesWindow.glade", "Window_Properties_GtkWindow", null);
        gxml.Autoconnect (this);

        // Now let's fill the values...
 
        /* Database page */
        DBInfo __DBInfo = MovieDB.GetDBInfo();
        Name_GtkEntry.Text = __DBInfo.Name;
        Owner_GtkEntry.Text = __DBInfo.Owner;
        Contact_GtkEntry.Text = __DBInfo.Contact;
        Comment_GtkTextView.Buffer.Text = __DBInfo.Comment;

        /* Network page */
        if ((bool) GConfRegistry.Get(GConfKeys.UseGNOMEBrowser, GConfKeys.UseGNOMEBrowser_Default)) BrowserGNOME_GtkRadioButton.Active = true;
        else BrowserCustom_GtkRadioButton.Active = true;
 
        if ((bool) GConfRegistry.Get(GConfKeys.UseGNOMEProxy, GConfKeys.UseGNOMEProxy_Default)) ProxyGNOME_GtkRadioButton.Active = true;
        else ProxyCustom_GtkRadioButton.Active = true; 
        BrowserCommand_GtkEntry.Text = (string) GConfRegistry.Get(GConfKeys.CustomBrowserCommand, GConfKeys.CustomBrowserCommand_Default);
        Server_GtkEntry.Text = (string) GConfRegistry.Get(GConfKeys.ProxyServer, GConfKeys.ProxyServer_Default);
        Port_GtkEntry.Text = ((int) GConfRegistry.Get(GConfKeys.ProxyPort, GConfKeys.ProxyPort_Default)).ToString();        
        Username_GtkEntry.Text = (string) GConfRegistry.Get(GConfKeys.ProxyUser, GConfKeys.ProxyUser_Default);        
        Password_GtkEntry.Text = (string) GConfRegistry.Get(GConfKeys.ProxyPassword, GConfKeys.ProxyPassword_Default);

        /* Display page */
        Util.Widgets.SetColorPickerFromHex(Borrowed_GnomeColorPicker, (string) GConfRegistry.Get(GConfKeys.BorrowedColor, GConfKeys.BorrowedColor_Default));
        Util.Widgets.SetColorPickerFromHex(OverBorrowed_GnomeColorPicker, (string) GConfRegistry.Get(GConfKeys.OverBorrowedColor, GConfKeys.OverBorrowedColor_Default));
        Util.Widgets.SetColorPickerFromHex(New_GnomeColorPicker, (string) GConfRegistry.Get(GConfKeys.NewColor, GConfKeys.NewColor_Default));
        
        NewDays_GtkEntry.Text = ((int) GConfRegistry.Get(GConfKeys.DaysNew, GConfKeys.DaysNew_Default)).ToString();
        OverBorrowedDays_GtkEntry.Text = ((int) GConfRegistry.Get(GConfKeys.DaysOverBorrowed, GConfKeys.DaysOverBorrowed_Default)).ToString();
        
        // Show
        
        Window_Properties_GtkWindow.ShowAll();
        Application.Run();

    }
    

    /* Delete Event */
    public void On_WindowDelete (object o, DeleteEventArgs args) 
    {
        KillWindow(false);
    }


    /* Default GNOME browser selected */
    public void On_BrowserGNOMEToggled(object o, EventArgs args)
    {
        // Disable controls
        BrowserCommand_GtkEntry.Sensitive = false;
    }

    
    /* Custom browser selected */
    public void On_BrowserCustomToggled(object o, EventArgs args)
    {
        // Enable controls
        BrowserCommand_GtkEntry.Sensitive = true;
    }

    
    /* Default GNOME prxy selected */
    public void On_ProxyGNOMEToggled(object o, EventArgs args)
    {
        // Disable controls
        Server_GtkEntry.Sensitive = false;
        Port_GtkEntry.Sensitive = false;
        Username_GtkEntry.Sensitive = false;
        Password_GtkEntry.Sensitive = false;

    }

    
    /* Custom browser selected */
    public void On_ProxyCustomToggled(object o, EventArgs args)
    {
        // Enable controls
        Server_GtkEntry.Sensitive = true;
        Port_GtkEntry.Sensitive = true;
        Username_GtkEntry.Sensitive = true;
        Password_GtkEntry.Sensitive = true;
        
    }


    /* Close button clicked */
    public void OnClose(object o, EventArgs args)
    {
        KillWindow(false);
    }


    /* Toggled when color is changed in any of the widgets */
    public void OnColorSet (object _Sender, ColorSetArgs _Args)
    {
        // Borrowed color
        if (_Sender == Borrowed_GnomeColorPicker) GConfRegistry.Set (GConfKeys.BorrowedColor, Util.Widgets.GetHexFromPicker(Borrowed_GnomeColorPicker));
        if (_Sender == OverBorrowed_GnomeColorPicker) GConfRegistry.Set (GConfKeys.OverBorrowedColor, Util.Widgets.GetHexFromPicker(OverBorrowed_GnomeColorPicker));
        if (_Sender == New_GnomeColorPicker) GConfRegistry.Set (GConfKeys.NewColor, Util.Widgets.GetHexFromPicker(New_GnomeColorPicker));
    }


    /* One of the timespan textentry values changed */
    public void OnDaysChanged(object _Sender, EventArgs _Args)
    {
        if (_Sender == NewDays_GtkEntry && Widgets.ValidateIntegerTextEntry(NewDays_GtkEntry)) GConfRegistry.Set (GConfKeys.DaysNew, Convert.ToInt32(NewDays_GtkEntry.Text));
        
        if (_Sender == OverBorrowedDays_GtkEntry && Widgets.ValidateIntegerTextEntry(OverBorrowedDays_GtkEntry)) GConfRegistry.Set (GConfKeys.DaysOverBorrowed, Convert.ToInt32(OverBorrowedDays_GtkEntry.Text));
    }


    /* Quite obvious */
    public void KillWindow(bool _ret)
    {
        // Save all the values
        
        /* Database */
        DBInfo __NewInfo = new DBInfo();
        DBInfo __OldInfo = MovieDB.GetDBInfo();
        
        __NewInfo.Name = Name_GtkEntry.Text;
        __NewInfo.Owner = Owner_GtkEntry.Text;
        __NewInfo.Contact= Contact_GtkEntry.Text;
        __NewInfo.Comment = Comment_GtkTextView.Buffer.Text;
        __NewInfo.Version = __OldInfo.Version;
        if (__OldInfo != __NewInfo) MovieDB.PutDBInfo(__NewInfo);

        /* Network */
        // Browser
        GConfRegistry.Set (GConfKeys.UseGNOMEBrowser, BrowserGNOME_GtkRadioButton.Active);
        GConfRegistry.Set (GConfKeys.CustomBrowserCommand, BrowserCommand_GtkEntry.Text);
        // Proxy
        GConfRegistry.Set (GConfKeys.UseGNOMEProxy, ProxyGNOME_GtkRadioButton.Active);
        GConfRegistry.Set (GConfKeys.ProxyServer, Server_GtkEntry.Text);
        GConfRegistry.Set (GConfKeys.ProxyPort, Convert.ToInt32(Port_GtkEntry.Text));
        GConfRegistry.Set (GConfKeys.ProxyUser, Username_GtkEntry.Text);
        GConfRegistry.Set (GConfKeys.ProxyPassword, Password_GtkEntry.Text);
        
        // Re-init the net
        Util.Net.Init();

        // Destroy window
        Window_Properties_GtkWindow.Destroy();
        Application.Quit ();
    }

}
