/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.Collections;
using Util;
using System.Data;
using Mono.Data.SqliteClient;


namespace MovieDatabase {

    public class DataCacheDefaults {

        public static string[] Genres = { "Action" , "Adult" , "Adventure" , "Animation" , "Comedy" ,
                                          "Crime" , "Documentary", "Drama" , "Fantasy" , "Film-Noir" , 
                                          "Horror" , "Music" , "Mystery" , "Romance" , "Sci-Fi" , 
                                          "Short" , "Theatre" , "Thriller" , "War" , "Western" };

        public static string[] Countries = { "Belgium" , "Bosnia-Herzegovina" , "Canada" , "Denmark" , "France" , 
                                             "Germany" , "Hong Kong" , "Iceland" , "Italy" , "Japan" ,  "Mexico" ,
                                             "New Zealand" , "Peru" , "Poland" , "Russia" , "Soviet Union" , "Spain" ,
                                             "Sweden" , "UK" , "USA" , "West Germany" };

        public static string[] VFormats = { "DivX" , "XviD" , "DVD" , "DivX 3.11 Low Motion" , 
                                            "DivX 3.11 High Motion" , "DivX 4.x", "DivX 5.x" , 
                                            "MPEG1" , "XviD MPEG-4 Codec" };

        public static string[] AFormats = { "AC3" , "DVD" , "AC3/MPEG Layer 3 (MP3)" , "DivX Audio (WMA)", 
                                            "MPEG Layer 3 (MP3)" };

        public static string[] Subtitles = { "ENG" , "PL" , "PL & ENG" };
        public static string[] Regions = { "0 - No Region Coding", "1 - USA, Canada" , "2 - Europe, Japan", 
                                           "3 - Korea, Indonesia, Vietnam", "4 - Australia, South America", 
                                           "5 - Russia, India, Africa", "6 - China", "8 - Airlines/Cruise Ships", 
                                           "9 - Expansion"  };
        
    }



    public class DataCache {

        // *** Members
        public ArrayList Values;
        string Name;
        
        // *** Contructor 
        public DataCache(string _Name) 
        {
            Name = _Name;
            
            // FIXME: We should move that to one 'check if table present' function'
            // Check if we have this table already in DB
            string __CheckQuery = String.Format("SELECT name FROM sqlite_master WHERE type='table' AND name='{0}'", Name);
            
            IDbCommand Cmd = MovieDB.DBConnection.CreateCommand();
            Cmd.CommandText = __CheckQuery;
            IDataReader Reader = Cmd.ExecuteReader();

            // FIXME: We should spare the sorting out! We should sort once and for all...
            
            if (Reader.Read()) {
                // Load from database
                Values = MovieDB.GetCompleteColumn(Name, "Value");
                SortValues();
            } else {
                // Create in DB 
                string __query = "CREATE TABLE " + Name + " (ID INTEGER PRIMARY KEY, Value TEXT)";
                MovieDB.ExecuteQuery(__query);
                Values = new ArrayList();
            }

            // clean up
            Reader.Close();
            Reader = null;
            Cmd.Dispose();
            Cmd = null;
            
            
        }


        // *** Ads value to cache/checks if present
        public bool AddValue(string _Val)
        {
            foreach (string _S in Values)
                if (_S == _Val) return false;

            Values.Add(_Val);
            // Add to DB
            string __query = "INSERT INTO " + Name + " (Value) VALUES ('" + Util.Strings.DBLize(_Val) + "')";
            Debug.Message("MovieDatabase.dll", "DataCache", String.Format("Adding new value to cache {0} , QUERY={1}", Name, __query));
            MovieDB.ExecuteQuery(__query);

            SortValues();
            return true;
        }

        public void FillFromStrings(string[] _Str)
        {
            foreach (string __S in _Str) 
                AddValue(__S);

        }

        
        public void SortValues()
        {
            Values.Sort();
        }
        
        // *** Debug function
        public void FlushToConsole()
        {
            foreach (string _S in Values)
                Console.WriteLine(_S);
        }

    }


}
