/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using PluginLib;
using MovieDatabase;
using System;
using Gtk;
using Glade;

public class CSVImport_Plugin : ImportPlugin
{
    bool Result = false;
    MovieDB DB;
    [Widget] Gtk.ScrolledWindow ColumnListContainer_GtkScrolledWindow;
    [Widget] Gtk.Button MoveUp_GtkButton;
    [Widget] Gtk.Button MoveDown_GtkButton;
    [Widget] Gtk.Window Window_Import_GtkWindow;
    [Widget] Gnome.FileEntry FileEntry_GnomeFileEntry;
    [Widget] Gtk.Entry Delimiter_GtkEntry;

    ToggleList FieldsList;

    public override int Initialize()
    {
        return 1;
    }

    public override string GetName()
    {
        return "Import from CSV";
    }

    public override string GetAuthor()
    {
        return "Michael Dominic K. <mdk@mdk.org.pl>";
    }

    public override string GetDescription()
    {
        return "A plugin to import information from plain text files (comma separated values, etc.).";
    }

    public override bool Run()
    {
        Glade.XML gxml = new Glade.XML (null, "CVSImport.glade", "Window_Import_GtkWindow", null);
        gxml.Autoconnect (this);
        FieldsList = new ToggleList();

        ColumnListContainer_GtkScrolledWindow.Add(FieldsList);
        FieldsList.ActivateMoveButtons(MoveUp_GtkButton, MoveDown_GtkButton);
        
        // Fill all the values
        int __i = 0;
        foreach (DataField _F in MovieDB.Fields)
            {
                if (_F.Importable) FieldsList.AppendItem(__i, false, _F.Name);
                __i++;
            }


        FieldsList.ShowAll();
        
        
        Application.Run();
        return Result;
    }

    public void OnWindowDeleteEvent (object o, DeleteEventArgs args) 
    {
        KillWindow(false);
    }

    public void KillWindow(bool _ret)
    {
        Window_Import_GtkWindow.Destroy();
        Result = _ret;
        Application.Quit ();
    }

    public void On_Cancel (object sender, EventArgs e)
    {
        KillWindow(false);
    }



    public void On_OK_Clicked (object sender, EventArgs e)
    {
        try 
            {
                                
                // Get the int reference
                int[] __Sel = FieldsList.CollectMapping();
                if (FileEntry_GnomeFileEntry.Filename == "") throw new Exception("You have to select a file!");
                if (Delimiter_GtkEntry.Text == "") throw new Exception("You have to enter a delimiting character!");
                Importer __Importer = new Importer(FileEntry_GnomeFileEntry.Filename, __Sel, Delimiter_GtkEntry.Text);
                __Importer.Run();
                __Importer.CloseFile();
                int __count = __Importer.ImportedMovies;
                
                if (__count == 0) throw new Exception("Not a single movie could be imported from the specified file! Please check if the column specification is right.");
                // Success
                MessageDialog md = new MessageDialog(Window_Import_GtkWindow, DialogFlags.DestroyWithParent, 
                                                     MessageType.Info, ButtonsType.Close, __count + 
                                                     " movies were imported into the database.");
                md.Run();
                md.Destroy();
                KillWindow(true);
                
                }
        catch (Exception excp) {
            // Fallback
            MessageDialog md = new MessageDialog(Window_Import_GtkWindow, DialogFlags.DestroyWithParent, 
                                                 MessageType.Warning, ButtonsType.Close, excp.Message);
            
            md.Run();
            md.Destroy();
            
            return;
        }
        
        
    }


    
}
