/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* This class creates/writes HTML according to given Template 
 */

using System;
using System.IO;
using MovieDatabase;
using Util;
using System.Collections;

public class HTMLWriter 
{
    HTMLSnippet HeaderSnippet;
    HTMLSnippet FooterSnippet;
    HTMLSnippet EvenRowSnippet;
    HTMLSnippet OddRowSnippet;
    
    StreamWriter Writer;
    bool Even;

    // Some variables for parsing...
    string DBName;
    string DBOwner;
    string DBContact;
    string DBComment;

    // Export settings
    string Charset;
    bool ExportOnlyNew = false;
    bool DoNotExportBorrowersName = false;
    string BaseFilename; 
    Int64 DaysNew; // How many days a movie is considered new
 
    // FIXME: We need to move all the settings to be accessible by all plugins
    // to a Util or something. Actually, we need to create a new module called MonothekaSettings
    // or something.
    
    ArrayList CreatedFiles = new ArrayList(); // Files created by this writer
    
  
    public HTMLWriter (Template _Template, string _Filename)
    {
        // Snippets
        _Template.SplitToSnippets(out HeaderSnippet, out FooterSnippet, out EvenRowSnippet, out OddRowSnippet);
        
        // Some info about the DB
        DBInfo __Info = MovieDB.GetDBInfo();
        DBName = __Info.Name;
        DBOwner = __Info.Owner;
        DBContact = __Info.Contact;
        DBComment = __Info.Comment;
        Charset = _Template.Charset;

        // IO
        Writer = new StreamWriter(_Filename, false, System.Text.Encoding.GetEncoding(Charset));
        CreatedFiles.Add(_Filename);

        // CSS file
        Debug.Message("HTMLExport.dll", "HTMLWriter", String.Format("Saving CSS file to: {0}", Path.GetDirectoryName(_Filename) + Path.DirectorySeparatorChar + Path.GetFileNameWithoutExtension(_Filename) + ".css"));
        if (_Template.StylesheetPresent) File.Copy(_Template.BaseFilename + ".css", Path.GetDirectoryName(_Filename) + Path.DirectorySeparatorChar + Path.GetFileNameWithoutExtension(_Filename) + ".css", true);
        CreatedFiles.Add(Path.GetDirectoryName(_Filename) + Path.DirectorySeparatorChar + Path.GetFileNameWithoutExtension(_Filename) + ".css");

        // We start with even row
        Even = true;

        // Get the days
        DaysNew = (int) GConfRegistry.Get(GConfKeys.DaysNew, GConfKeys.DaysNew_Default);

        // Base filename
        BaseFilename = Path.GetFileNameWithoutExtension(_Filename);
    }

    
    /* Parse the snippet with some basic params common to the database, etc */
    public HTMLSnippetParser ParseSnippetWithBasicParams (HTMLSnippetParser _PSnippet)
    {
        _PSnippet.Parse("@DATE@", (DateTime.Now).ToShortDateString());
        _PSnippet.Parse("@TIME@", (DateTime.Now).ToShortTimeString());
        _PSnippet.Parse("@DB_OWNER@", DBOwner);
        _PSnippet.Parse("@DB_NAME@", DBName);
        _PSnippet.Parse("@DB_CONTACT@", DBContact);
        _PSnippet.Parse("@DB_COMMENT@", DBComment);
        _PSnippet.Parse("@CHARSET@", Charset);
        _PSnippet.Parse("@CSSFILE@", BaseFilename + ".css");
        return _PSnippet;
    }
    
    
    /* Write the HTML header */
    public void WriteHeader()
    {
        HTMLSnippetParser __PSnippet = new HTMLSnippetParser(HeaderSnippet);
        __PSnippet = ParseSnippetWithBasicParams(__PSnippet);
        
        Writer.Write(__PSnippet.GetParsedHTML());
    }


    /* Write the HTML footer */
    public void WriteFooter()
    {
        HTMLSnippetParser __PSnippet = new HTMLSnippetParser(FooterSnippet);
        __PSnippet = ParseSnippetWithBasicParams(__PSnippet);
        
        Writer.Write(__PSnippet.GetParsedHTML());
    }


    /* Write movie to HTML */
    public object WriteMovie(Movie _M, object _NotImportant)
    {
        // Check if new
        if (ExportOnlyNew) {
            // Constants
            Int64 __Second = 10000000;
            Int64 __Day = 3600 * 24;

            // Days 
            Int64 __DaysNew = (Int64) 7;   
            
            Int64 __NewTS = DateTime.Now.ToFileTime() - (__Second * __Day * __DaysNew);
        
            // And check...
            if (Convert.ToInt64(_M.Data[(int)MovieField.ADDEDDATE])  < __NewTS) return null;
        }
               
        // Borrowers Name
        if (DoNotExportBorrowersName && _M.Data[(int)MovieField.BORROWER] != String.Empty) _M.Data[(int)MovieField.BORROWER] = "Yes";
        

        // Start snippetting
        HTMLSnippetParser __PSnippet;
        
        // Check row
        if (Even || OddRowSnippet == null) {
            __PSnippet = new HTMLSnippetParser(EvenRowSnippet);
        } else {
            __PSnippet = new HTMLSnippetParser(OddRowSnippet);
        }

        // Parse
        __PSnippet = ParseSnippetWithBasicParams(__PSnippet);

        // Now let's start replacing... ;)
        __PSnippet.Parse("@MOVIE_ID@", _M.Data[(int)MovieField.ID]);
        __PSnippet.Parse("@MOVIE_NO@", _M.Data[(int)MovieField.NO]);
        __PSnippet.Parse("@MOVIE_TITLE@", _M.Data[(int)MovieField.TITLE]);
        __PSnippet.Parse("@MOVIE_TRANSLATEDTITLE@", _M.Data[(int)MovieField.TRANSLATEDTITLE]);
        __PSnippet.Parse("@MOVIE_DIRECTOR@", _M.Data[(int)MovieField.DIRECTOR]);
        __PSnippet.Parse("@MOVIE_TIME@", _M.Data[(int)MovieField.TIME]);
        __PSnippet.Parse("@MOVIE_YEAR@", _M.Data[(int)MovieField.YEAR]);
        __PSnippet.Parse("@MOVIE_COUNTRY@", _M.Data[(int)MovieField.COUNTRY]);
        __PSnippet.Parse("@MOVIE_GENRE@", _M.Data[(int)MovieField.GENRE]);
        __PSnippet.Parse("@MOVIE_DISCS@", _M.Data[(int)MovieField.DISCS]);
        __PSnippet.Parse("@MOVIE_VIDEOFORMAT@", _M.Data[(int)MovieField.VIDEOFORMAT]);
        __PSnippet.Parse("@MOVIE_AUDIOFORMAT@", _M.Data[(int)MovieField.AUDIOFORMAT]);
        __PSnippet.Parse("@MOVIE_QUALITY@", _M.Data[(int)MovieField.QUALITY]);
        __PSnippet.Parse("@MOVIE_SUBTITLES@", _M.Data[(int)MovieField.SUBTITLES]);
        __PSnippet.Parse("@MOVIE_RELEASE@", _M.Data[(int)MovieField.RELEASE]);
        __PSnippet.Parse("@MOVIE_BORROWER@", _M.Data[(int)MovieField.BORROWER]);
        __PSnippet.Parse("@MOVIE_DESCRIPTION@", _M.Data[(int)MovieField.DESCRIPTION]);
       
        // Added date. Theoretically should always be
        // != String.Empty
        // 
        if (_M.Data[(int)MovieField.ADDEDDATE] != String.Empty) __PSnippet.Parse("@MOVIE_ADDEDDATE@", DateTime.FromFileTime(Convert.ToInt64(_M.Data[(int)MovieField.ADDEDDATE])).ToShortDateString()); else
            __PSnippet.Parse("@MOVIE_ADDEDDATE@", String.Empty);
        
        // Borrowed date        
        if (_M.Data[(int)MovieField.BORROWEDDATE] != String.Empty) __PSnippet.Parse("@MOVIE_BORROWEDDATE@", DateTime.FromFileTime(Convert.ToInt64(_M.Data[(int)MovieField.BORROWEDDATE])).ToShortDateString()); else
            __PSnippet.Parse("@MOVIE_BORROWEDDATE@", String.Empty);
        
        __PSnippet.Parse("@MOVIE_DVDREGION@", _M.Data[(int)MovieField.DVDREGION]);
        Writer.Write(__PSnippet.GetParsedHTML());

        // Alternate rows
        Even = ! Even;

        return null;
    }
    

    /* Close the stream */
    public void Close()
    {
        if (Writer != null) Writer.Close();
        Writer = null;
    }


    /* Destructor */
    ~HTMLWriter()
    {
        Close();
    }


    /* Delete all the files created by this writer. Needed to erase preview files */
    public void DeleteFiles()
    {
        foreach (string __File in CreatedFiles) {
            try {
                Debug.Message("HTMLExport.dll", "HTMLWriter", String.Format("Deleting created file {0}", __File));
                File.Delete(__File);
            } catch (Exception __Excp) {
                // FIXME: Do we really need this?
                //new Util.Windows.OKDialogWindow(__Excp.Message, null, MessageType.Error);
            }
        }
           
    }

   
    /* Set the flag for the exporter */
    public void Set_DoNotExportBorrowersName()
    {
        DoNotExportBorrowersName = true;
    }


    public void Set_ExportOnlyNew()
    {
        ExportOnlyNew = true;
    }
}
