/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using Gtk;
using Glade;

namespace Util.Windows {
        
    public abstract class ListWindow : Window {
        
        [Widget] Window Window_List_GtkWindow;
        [Widget] Label Text_GtkLabel;
        [Widget] TreeView Tree_GtkTreeView;
        [Widget] Button Ok_GtkButton;
        [Widget] Button Cancel_GtkButton;
        [Widget] Image Brand_GtkImage;

        TreeStore Store;
        CellRendererText TextRenderer = new CellRendererText();

        /* Construct with a image */
        public ListWindow(string _Text, string _Title, bool _SecondColumnVisible, int _Height, Gdk.Pixbuf _PxBuf) : this(_Text, _Title, _SecondColumnVisible, _Height) 
        {
            Brand_GtkImage.Pixbuf = _PxBuf;
        }

        
        /* Constructor */
        public ListWindow(string _Text, string _Title, bool _SecondColumnVisible, int _Height) : base (IntPtr.Zero)
        {
            Glade.XML gxml = new Glade.XML (null, "ListWindow.glade", "Window_List_GtkWindow", null);
            gxml.Autoconnect (this);
            
            Raw = Window_List_GtkWindow.Handle;
            HeightRequest = _Height;
            
            // # Initialize components
            Text_GtkLabel.Text = _Text;
            Title = _Title;
            
            //    Treestore
            Store = new TreeStore (typeof(string), typeof(string));
            Tree_GtkTreeView.Model = Store;
            
            //    Columns
            Tree_GtkTreeView.AppendColumn ("First", TextRenderer, "text", 0);
            TreeViewColumn __SecondClmn = Tree_GtkTreeView.AppendColumn ("Second", TextRenderer, "text", 0);
            __SecondClmn.Visible = _SecondColumnVisible;
        }
        
        
        /* Ads the item to the tree */
        public void AddItem(string _1Column, string _2Column)
        {
            TreeIter __Row = Store.AppendValues();
            Store.SetValue(__Row, 0, _1Column);
            Store.SetValue(__Row, 1, _2Column);

        }


        /* Fired when user selects any value */
        public void On_Selection(object sender, EventArgs e)
        {
            Ok_GtkButton.Sensitive = true;
        }

        
        /* Double clicking a row 
           OR clicking the OK button */
        public void On_RowActivated(object sender, EventArgs e)
        {
            string __1st, __2nd;
            GetSelectedValues(out __1st, out __2nd);
        }

        
        /* DestroyWindow */
        protected virtual void DestroyWindow()
        {
            Destroy();
        }


        /* When user clicks the close */
        protected void On_WindowDelete(object o, DeleteEventArgs args) 
        {
            DestroyWindow();
        }

        
        /* Cancel clicked */
        public void On_Cancel(object sender, EventArgs e)
        {
            DestroyWindow();
        }
        
        /* get the Columns of the selected row */
        public void GetSelectedValues(out string _1st, out string _2nd)
        {
            // FIXME: Perhaps we can make it a bit simplier...
            TreeIter SelectedIter;
            TreeSelection clicked = Tree_GtkTreeView.Selection;
            TreeModel model; // We don't need that, do we? 
            clicked.GetSelected(out model , out SelectedIter);
            
            _1st = (string) Store.GetValue(SelectedIter, 0);
            _2nd = (string) Store.GetValue(SelectedIter, 1);
            
            ItemTriggered(_1st, _2nd);

        }

        /* Called when the item is selected via doubble click or OK
           button. */
        protected abstract void ItemTriggered(string _1st, string _2nd);

    }

}
