/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using System;
using System.Net;
using Util;
using Util.Windows;
using Gtk;
using System.Threading;


namespace Util {

    public class NetworkException : ApplicationException {
        
        public NetworkException(string _Msg) : base (_Msg)
        {
        }

    }

    public class ThreadedWebClient {
        
        WebClient WC = new WebClient();
        Exception Excp = null;
        byte[] Response;
        int Timeout = 60000;
        string Url = null;
        
         
        private void ThreadStarter()
        {
            try {
                Response = WC.DownloadData(Url);
            }
            catch (Exception __Excp) {
                Excp = __Excp;
            }
        }
        
        public byte[] DownloadData(string _Url)
        {
            Url = _Url;
            Excp = null;

            Thread WebT = new Thread(new ThreadStart(ThreadStarter));
            WebT.Start();

            if (WebT.Join(Timeout)) {
                // Managed to finish
                if (Excp != null) throw Excp;
                else return Response;
            } else {
                WebT.Abort();
                throw new Exception("Connection timeout!");
            }            
            
        }
        

    }



    public class Net {

        

        private static readonly string ___ConnectionException = "The network connection with host {0} could not be established! Check your proxy/internet settings.";
        private static readonly string ___GnomeBrowserError = "Can't launch default GNOME web browser! Check Monotheka network settings and GNOME default applications settings.";
        private static readonly string ___CustomBrowserError = "Can't launch the custom web browser you specified! Check Monotheka network settings.";

        // Use Gnome Proxy 
        private const string GConf_UseGNOMEProxy = "/apps/monotheka/network/UseGnomeProxy" ;

        // Gnome proxy keys
        private const string GConf_GNOMEProxyServer = "/system/http_proxy/host";
        private const string GConf_GNOMEProxyPort = "/system/http_proxy/port";
        private const string GConf_GNOMEProxyUsername = "/system/http_proxy/authentication_user";
        private const string GConf_GNOMEProxyPassword = "/system/http_proxy/authentication_password";
        private const string GConf_GNOMEUseHttpProxy = "/system/http_proxy/use_http_proxy";
        private const string GConf_GNOMEUseAuthentication = "/system/http_proxy/use_authentication";

        // Custom proxy keys
        private const string GConf_ProxyServer = "/apps/monotheka/network/ProxyServer" ;
        private const string GConf_ProxyPort = "/apps/monotheka/network/ProxyPort" ;
        private const string GConf_ProxyUsername = "/apps/monotheka/network/ProxyUser" ;
        private const string GConf_ProxyPassword = "/apps/monotheka/network/ProxyPassword" ;
        
        // Use Gnome browser
        private const string GConf_UseGNOMEBrowser = "/apps/monotheka/network/UseGnomeBrowser" ;
        
        // Custom browser commands
        private const string GConf_CustomBrowserCommand = "/apps/monotheka/network/CustomBrowser" ;
        private const string GConf_CustomBrowserCommand_Default = "mozilla" ;

        
        // MEMBERS
        static ThreadedWebClient WC = null;
            
        /* Constructor */
        static Net()
        {
            Init();
        }

        
        /* Init the WebClient according to GConf settings */
        public static void Init()
        {
            bool __UseGNOMEProxy = (bool) GConfRegistry.Get(GConf_UseGNOMEProxy, true);

            string __ProxyServer;
            int __ProxyPort;
            string __ProxyUsername, __ProxyPassword;
            bool __UseProxy = false, __UseCredentials = false;
               

            if (__UseGNOMEProxy == true) {
                __ProxyServer = (string) GConfRegistry.Get(GConf_GNOMEProxyServer, "");
                __ProxyPort = (int) GConfRegistry.Get(GConf_GNOMEProxyPort, 8080);
                __ProxyUsername = (string) GConfRegistry.Get(GConf_GNOMEProxyUsername, "");
                __ProxyPassword = (string) GConfRegistry.Get(GConf_GNOMEProxyPassword, "");
                
                __UseProxy = (bool) GConfRegistry.Get(GConf_GNOMEUseHttpProxy, false);
                __UseCredentials = (bool) GConfRegistry.Get(GConf_GNOMEUseAuthentication, false);
            } else {
                __ProxyServer = (string) GConfRegistry.Get(GConf_ProxyServer, "");
                __ProxyPort = (int) GConfRegistry.Get(GConf_ProxyPort, 8080);
                __ProxyUsername = (string) GConfRegistry.Get(GConf_ProxyUsername, "");
                __ProxyPassword = (string) GConfRegistry.Get(GConf_ProxyPassword, "");

                if (__ProxyServer != "") __UseProxy = true;
                if (__ProxyUsername != "") __UseCredentials = true;
            }
            
            if ( __UseProxy ) {
                // Set proxy server
                WebProxy __PR = new WebProxy( __ProxyServer, __ProxyPort);
                if ( __UseCredentials ) {
                    // Set credentials
                    NetworkCredential __NC = new NetworkCredential(__ProxyUsername, __ProxyPassword);
                    __PR.Credentials = __NC;
                }
                // Set it
                GlobalProxySelection.Select = (IWebProxy) __PR;
            } else {
                // Reset the proxy
                GlobalProxySelection.Select = (IWebProxy) GlobalProxySelection.GetEmptyWebProxy();
            }
          
            WC = new ThreadedWebClient();
        }
        
        
        /* Launches the default webbrowser */
        public static void LaunchWebbrowser(string _Url)
        {
            bool __UseGNOMEBrowser = (bool) GConfRegistry.Get(GConf_UseGNOMEBrowser, true);
            if (__UseGNOMEBrowser) {
                try {
                    Gnome.Url.Show(_Url);
                }
                catch {
                    throw new NetworkException(___GnomeBrowserError);
                }
            } 
            else {
                // Custom command
                string __Command = (string) GConfRegistry.Get(GConf_CustomBrowserCommand, GConf_CustomBrowserCommand_Default);
                __Command = Strings.ClearString(__Command);
                
                // parse url
                _Url = _Url.Replace(" ", "%20");
                                
                try {
                    System.Diagnostics.Process proc = new System.Diagnostics.Process(); 
                    proc.EnableRaisingEvents = false; 
                    proc.StartInfo.FileName = __Command; 
                    proc.StartInfo.Arguments = _Url;
                    proc.Start(); 
                }
                catch {
                    throw new NetworkException(___CustomBrowserError);
                }
                    
            }
            
            
            
            


        }
        
        /* Download a given http webpage and return it as HTML string */
        public static string GetWebpageAsString(string _Url, string _Charset)
        {
            try {
                byte[] __Data = WC.DownloadData(_Url);
                string __WWW = Strings.ANSIBytesToUTF8String(__Data, _Charset);
                return Tags.RemoveHTMLEntities(__WWW, _Charset);
            }
            catch (Exception __Excp) {
                throw new NetworkException(__Excp.Message + "\n" + String.Format(___ConnectionException, _Url));
            }
            
        }
        
        

        /* Download a given http webpage and return it as HTML string */
        public static string GetWebpageAsString(string _Url)
        { 
            return GetWebpageAsString(_Url, "ISO-8859-2");
        }
        
        
    }

}
    
